// general header file for HW libraries (for run-time linking)
//
#pragma once

#include "common.h"

#define BITS_PER_CHIP           917512      // number of bits for one chip readout (256*256*14 + 8)
#define BITS_PER_CHIP_MXR       917768      // number of bits for one chip readout (256*256*14 + 8 + 256)
#define BYTES_PER_CHIP          114689      // number of bytes for one chip readout
#define BYTES_PER_CHIP_MXR     (114689+32)  // number of bytes for one chip readout
#define CLOCK_RESET             28680       // number of cycle clock to perform reset
#define CLOCK_RESET_MXR         28688       // number of cycle clock to perform reset (8preload+256x8x14+8postload)


typedef struct _MpxHwFuncs MpxHwFuncs;
typedef struct _MpxInfo MpxInfo;
typedef enum _DACS_CODES DACS_CODES;

// for runtime linking of HW library call function getMandatoryFuncs() of type GetMandatoryFuncsType
// to obtain table of functions which every HW library has to provide
typedef MpxHwFuncs* (*GetMandatoryFuncsType)();

typedef void (*HwCallback)(int eventType, void *data);

typedef int (*HwInitFirst)(int *hwID);
typedef int (*HwInitNext)(int *hwID);
typedef MpxInfo* (*HwGetMpxInfoType)(int hwID);
typedef void (*HwSetCallbackType)(int hwID, HwCallback callback);
typedef int (*HwGetHwInfoCountType)();
typedef int (*HwGetHwInfoType)(int hwID, int index, HwInfoItem *hwInfo, int *dataSize);
typedef int (*HwSetHwInfoType)(int hwID, int index, void *data, int dataSize);

// starts aquisition, seconds is number of seconds if acquisition is controlled by muros timer (mpxInfo->useHwTimer is TRUE)
//
typedef int (*HwStartAcquisitionType)(int hwID, double seconds);

// stops aquisition
//
typedef int (*HwStopAcquisitionType)(int hwID);

// frameCount - number of frames, seconds - if no external shutter is used, number of seconds for muros timer
// buff - buffer of length buffLen to hold all frames
typedef int (*HwStartContAcqType)(int hwID, u16 frameCount, double seconds, i16 *buff, u32 buffLen);

// abort continuous acquisition
typedef int (*HwAbortContAcqType)(int hwID);

// check state of running continuous acquisiton, return number of data samples (i16) which remain to be read
typedef int (*HwCheckContAcqProgressType)(int hwID, u32 *remain);

// lowVal is low values of the pulse, highVal is high values of pulse, pulseLength in clock cycles, pulseCount is number of pulses
// 
typedef int (*HwTestPulsesType)(int hwID, double lowVal, double highVal, byte pulseLength, u16 pulseCount);

// dacVals is array of DAC values (number of chips)*13, order of DACs is given by enum type DACS_ORDER, 
// codes is [number of chip] array which contains DAC codes for individual chips for ADC conversion/external DAC selection
// senseChip is 8bit mask for enabling/disabling preparation of ADC conversion of DAC selected in codes array
// extDacChip is 8bit mask for enabling/disabling usage of external DAC for DAC selected in codes array
// tpReg (for setting test pulse register in MXR, ignored for original chip)
typedef int (*HwSetMpxDACsType)(int hwID, DACTYPE dacVals[], int size, byte senseChip, byte extDacChip, int codes[], u32 tpReg);

// read-back analog value from ADC conversion, from chip chipNumber, output value is stored in value [Volt]
// DAC which is read is set by DAC code in codes array, mpxSetMpxDACs
typedef int (*HwGetMpxDacValType)(int hwID, int chipNumber, double *value);

// set pixels mask, cfgs is array [number of chips][65536]
typedef int (*HwSetPixelsCfgType)(int hwID, PixelCfg cfgs[][MATRIX_SIZE]);

// reset pixel matrix
typedef int (*HwResetMatrixType)(int hwID);

// bit-stream readout, size of buffer has to be at least (BYTES_PER_CHIP*numberOfChips)/2 round up to 2bytes
typedef int (*HwReadMatrixType)(int hwID, i16 *buff, u32 buffSize);

// write bit stream to medipix, size of buffer has to be at least (BYTES_PER_CHIP*numberOfChips)/2 round up to 2bytes
typedef int (*HwWriteMatrixType)(int hwID, i16 *buff, u32 buffSize);

// value is output value of the DAC in range <0,5> [V]
typedef int (*HwSetBiasType)(int hwID, double value);

// resets interface and medipix to default state
typedef int (*HwGeneralResetType)(int hwID);

// return string describing last error

typedef const char* (*HwGetLastErrorGeneralType)();

// return string describing last error for HW identified by hwID
typedef const char* (*HwGetLastErrorType)(int hwID);

// return state of busy line
typedef int (*HwCheckBusyType)(int hwID, i16 *state);
typedef int (*HwCloseDeviceType)(int hwID);

// conversion functions
//
typedef int (*BitStream2DataType)(int hwID, const i16* stream, u32 streamLen, i16* data, u32 dataLen);
typedef int (*Data2BitStreamType)(int hwID, const i16* data, u32 dataLen, i16* stream, u32 streamLen);
typedef int (*ConvertValuesType)(int hwID, i16 values[], int length, BOOL fromPseudo);


typedef struct _MpxHwFuncs
{
    int size;

    HwInitFirst hwInitFirst;
    HwInitNext hwInitNext;
    HwGetMpxInfoType hwGetMpxInfo;
    HwSetCallbackType hwSetCallback;
    HwGetHwInfoCountType hwGetHwInfoCount;
    HwGetHwInfoType hwGetHwInfo;
    HwSetHwInfoType hwSetHwInfo;

    HwStartAcquisitionType hwStartAcquisition;
    HwStopAcquisitionType hwStopAcquisition;
    HwStartContAcqType hwStartContAcq;
    HwAbortContAcqType hwAbortContAcq;
    HwCheckContAcqProgressType hwCheckContAcqProgress;
    HwTestPulsesType hwTestPulses;
    HwSetMpxDACsType hwSetMpxDACs;
    HwGetMpxDacValType hwGetMpxDacVal;
    HwSetPixelsCfgType hwSetPixelsCfg;
    HwResetMatrixType hwResetMatrix;
    HwReadMatrixType hwReadMatrix;
    HwWriteMatrixType hwWriteMatrix;
    HwSetBiasType hwSetBias;
    HwGeneralResetType hwGeneralReset;
    HwGetLastErrorGeneralType hwGetLastErrorGeneral;

    HwGetLastErrorType hwGetLastError;
    HwCheckBusyType hwCheckBusy;
    HwCloseDeviceType hwCloseDevice;

    BitStream2DataType bitStream2Data;
    Data2BitStreamType data2BitStream;
    ConvertValuesType convertValues;
} MpxHwFuncs;


typedef struct _MpxInfo
{
    // aqusition settings
    //
    BOOL polarityPositive;
    BOOL enableCst;
    BOOL useExternalShutter;
    BOOL useHwTimer;	

    // medipix parameters
    //
    int numberOfChips;
    int numberOfRows;
    BOOL okFSR;
    BOOL okSpareFSR;
    BOOL useSpareFSR;

    // additional info
    BOOL isMXR;
    char chipboardID[MPX_MAX_CHBID];
    const char *ifaceName;
} MpxInfo;
